@@ -4,6 +4,7 @@ import logging
 from glob import glob
 import spacy
 from spacy import displacy
+from spacy.matcher import PhraseMatcher
 import pandas as pd
 from bs4 import BeautifulSoup
 from tqdm import tqdm
@@ -14,7 +15,10 @@ from pathlib import Path
 from pygetpapers import Pygetpapers
 from collections import Counter
 import pip
+import json
+import re
 from lxml import etree
+import pyparsing as pp
 from pygetpapers.download_tools import DownloadTools
 from urllib.request import urlopen
 def install(package):
@@ -23,7 +27,7 @@ def install(package):
     else:
         pip._internal.main(['install', package])
 
-
+#nlp_phrase = spacy.load("en_core_web_sm")
 
 class EntityExtraction:
     """ """
@@ -94,7 +98,7 @@ class EntityExtraction:
 
     def extract_entities_from_papers(self, corpus_path, terms_xml_path, search_section,entities,query=None, hits=30,
                                      run_pygetpapers=False, make_section=False, removefalse=True, 
-                                     csv_name=False, make_ami_dict=False,spacy_model=False,html_path=False, synonyms=False):
+                                     csv_name=False, make_ami_dict=False,spacy_model=False,html_path=False, synonyms=False, make_json=False):
         self.spacy_model=spacy_model                             
         corpus_path=os.path.abspath(corpus_path)
         if run_pygetpapers:
@@ -119,9 +123,9 @@ class EntityExtraction:
             self.remove_statements_not_having_xmldict_entities(
                         dict_with_parsed_xml=self.sentence_dictionary)
         if terms_xml_path:
-            terms = self.get_terms_from_ami_xml(terms_xml_path)
+            compiled_terms = self.get_terms_from_ami_xml(terms_xml_path)
             self.add_if_file_contains_terms(
-                terms=terms, dict_with_parsed_xml=self.sentence_dictionary)
+                compiled_terms=compiled_terms, dict_with_parsed_xml=self.sentence_dictionary)
             if removefalse:
                 self.remove_statements_not_having_xmldict_terms(
                     dict_with_parsed_xml=self.sentence_dictionary)
@@ -137,6 +141,8 @@ class EntityExtraction:
         if csv_name:
             self.convert_dict_to_csv(
                 path=os.path.join(corpus_path, f'{csv_name}'), dict_with_parsed_xml=self.sentence_dictionary)
+        if make_json:
+            self.convert_dict_to_json(path=os.path.join(corpus_path, f'{make_json}'), dict_with_parsed_xml=self.sentence_dictionary)
         if make_ami_dict:
             ami_dict_path = os.path.join(corpus_path,make_ami_dict)
             self.handle_ami_dict_creation(self.sentence_dictionary,ami_dict_path)
@@ -235,16 +241,38 @@ class EntityExtraction:
             abbreviation_start.append(abrv.start)
             abbreviation_end.append(abrv.end)
 
-    def add_if_file_contains_terms(self, terms, dict_with_parsed_xml, searching='terms'):
+    def add_if_file_contains_terms(self, compiled_terms, dict_with_parsed_xml, searching='terms'):
 
-        for statement in dict_with_parsed_xml:
+        for statement in tqdm(dict_with_parsed_xml):
             dict_for_sentence = dict_with_parsed_xml[statement]
             dict_for_sentence[f'has_{searching}'] = []
-            for term in terms:
-                if term.lower().strip() in dict_for_sentence['sentence'].lower():
-                    dict_for_sentence[f'has_{searching}'].append(term)
-            dict_for_sentence[f'weight_{searching}'] = len(
-                dict_for_sentence[f'has_{searching}'])
+            term_list, frequency = self.search_sentence_with_compiled_terms(compiled_terms, dict_for_sentence['sentence'])
+            if term_list:
+                dict_for_sentence[f'has_{searching}'].append(term_list)
+                dict_for_sentence[f'weight_{searching}'] = frequency
+        
+    def is_phrase_in(self, phrases, text):
+        token_list = []
+        text = text.lower()
+        for phrase in phrases: 
+            phrase.lower().strip()
+            rule = pp.ZeroOrMore(pp.Keyword(phrase))
+            for token, start, end in rule.scanString(text):
+                if token:
+                    token_list.append(token[0])
+            frequency = (len(token_list))
+        return token_list, frequency
+    
+    def search_sentence_with_compiled_terms(self, compiled_terms, sentence):
+        # https://stackoverflow.com/questions/47681756/match-exact-phrase-within-a-string-in-python
+        match_list = []
+        frequency = 0
+        for compiled_term in compiled_terms:
+            term_match = compiled_term.search(sentence, re.IGNORECASE)
+            if term_match is not None:
+                match_list.append(term_match.group())
+            frequency = len(match_list)
+        return match_list, frequency
 
     def get_terms_from_ami_xml(self, xml_path):
         if xml_path in self.dict_of_ami_dict.keys():
@@ -256,10 +284,18 @@ class EntityExtraction:
             logging.info(f"getting terms from {xml_path}")
             tree = ET.parse(xml_path)
             root = tree.getroot()
-            terms = []
-            for para in root.iter('entry'):
-                terms.append(para.attrib["term"])
-            return terms
+            compiled_terms = []
+        for para in root.iter('entry'):
+            term = (para.attrib["term"])
+            try: 
+                compiled_term = self.regex_compile(term)
+            except re.error:
+                print(f'cannot use term {term}')
+            compiled_terms.append(compiled_term)
+        return (compiled_terms)
+    
+    def regex_compile(self, term):
+        return re.compile(r'\b{}\b'.format(term))
 
     def get_synonyms_from_ami_xml(self, xml_path):
         if xml_path in self.dict_of_ami_dict.keys():
@@ -321,6 +357,13 @@ class EntityExtraction:
         df.to_csv(path, encoding='utf-8', line_terminator='\r\n')
         logging.info(f"wrote output to {path}")
 
+    def convert_dict_to_json(self, path, dict_with_parsed_xml):
+        with open(path ,mode='w', encoding='utf-8') as f:
+             json.dump(dict_with_parsed_xml, f, indent = 4)
+        logging.info(f"wrote JSON output to {path}")
+        
+
+
     def remove_statements_not_having_xmldict_terms(self, dict_with_parsed_xml, searching='terms'):
         statement_to_pop = []
         for statement in dict_with_parsed_xml:
